#!/bin/bash

set -e

CMD="$@"

# check if docker installed and running
docker info &>/dev/null || (echo "WARNING: Distribution requires Docker daemon UP and running" && exit 1)

DISTRIBUTION_VERSION="1.0.0"
DOCKER_SERVER_CURRENT_VER=$(docker version --format '{{.Server.Version}}' 2>/dev/null) || DOCKER_SERVER_CURRENT_VER=""

# Distribution installer workdir
DISTRIBUTION_DIR="${HOME}/.jfrog/distribution/distribution-installer"

# Distribution installer conf file, should be created in the first run
DISTRIBUTION_CONF="${DISTRIBUTION_DIR}/distribution-env.conf"

DISTRIBUTION_MOUNT_ROOT_DEFAULT="${HOME}/.jfrog/distribution"

[ ! -d "$DISTRIBUTION_DIR" ] && mkdir -p "$DISTRIBUTION_DIR"

# Create the installer configuration file
if [ ! -f "$DISTRIBUTION_CONF" ]; then
    echo -e "\nDISTRIBUTION_MOUNT_ROOT must be set on distribution first installation \nPlease type desired DISTRIBUTION_MOUNT_ROOT location"

    read -p "Enter mount root [${DISTRIBUTION_MOUNT_ROOT_DEFAULT}]: " DISTRIBUTION_MOUNT_ROOT
    DISTRIBUTION_MOUNT_ROOT=${DISTRIBUTION_MOUNT_ROOT:-${DISTRIBUTION_MOUNT_ROOT_DEFAULT}}

    # Write to conf file
    echo "export DISTRIBUTION_MOUNT_ROOT=${DISTRIBUTION_MOUNT_ROOT}" >> ${DISTRIBUTION_CONF}
    echo "DISTRIBUTION_MOUNT_ROOT is now specified in ${DISTRIBUTION_CONF}, as ${DISTRIBUTION_MOUNT_ROOT}"
fi

# Load the installer configuration file
if [ -f "$DISTRIBUTION_CONF" ]; then
    source "${DISTRIBUTION_CONF}"
else
    echo "ERROR: configuration file does not exist [${DISTRIBUTION_CONF}]"
    exit 1
fi

if [ -z ${DISTRIBUTION_MOUNT_ROOT} ]; then
    echo "ERROR: DISTRIBUTION_MOUNT_ROOT is not set, please update the conf file [${DISTRIBUTION_CONF}]"
    exit 1
fi
echo "INFO: Using DISTRIBUTION_MOUNT_ROOT=${DISTRIBUTION_MOUNT_ROOT}"

# Setup options for connecting to docker host
if [ -z "$DOCKER_HOST" ]; then
    DOCKER_HOST="/var/run/docker.sock"
fi
if [ -S "$DOCKER_HOST" ]; then
    DOCKER_ADDR="-v $DOCKER_HOST:$DOCKER_HOST -e DOCKER_HOST"
else
    DOCKER_ADDR="-e DOCKER_HOST -e DOCKER_TLS_VERIFY -e DOCKER_CERT_PATH"
fi

# Only allocate tty if we detect one
if [ -t 1 ]; then
    DOCKER_RUN_OPTIONS="-t"
fi
if [ -t 0 ]; then
    DOCKER_RUN_OPTIONS="$DOCKER_RUN_OPTIONS -i"
fi

DISTRIBUTION_VOLUMES="-v ${DISTRIBUTION_MOUNT_ROOT}/distribution-installer:/data -v ${DISTRIBUTION_MOUNT_ROOT}:/DISTRIBUTION_GLOBAL_MOUNT_ROOT"

## Special configuration ##
## Set DISTRIBUTION_LOCAL_DOCKER_REPO in DISTRIBUTION_CONF to use local docker repo
DISTRIBUTION_DOCKER_REPO=${DISTRIBUTION_LOCAL_DOCKER_REPO:-"jfrog-docker-enterprise-docker.bintray.io"}
DISTRIBUTIONDB_DOCKER_REPO=${DISTRIBUTION_LOCAL_DOCKER_REPO:-"docker.bintray.io/jfrog"}
## Set DOCKER_INSTALLER_OFFLINE=true, to use local docker registry, then you do not need to pull
DOCKER_INSTALLER_OFFLINE=${DOCKER_INSTALLER_OFFLINE:-"false"}
## Set DOCKER_CONFIG_PATH, docker json-conf file location, e.g.
DOCKER_CONFIG_PATH=${DOCKER_CONFIG_PATH:-"false"}
if [ ! -z "${DOCKER_CONFIG_PATH}" ] && [ ! "${DOCKER_CONFIG_PATH}" == "false" ] && [ -f "$(ls ${DOCKER_CONFIG_PATH})" ]; then
    echo "Using host docker authentication ..."
    DOCKER_CONFIG_PATH="$(ls ${DOCKER_CONFIG_PATH})"
    if [ -f "${DOCKER_CONFIG_PATH}" ]; then
        DISTRIBUTION_VOLUMES="${DISTRIBUTION_VOLUMES} -v ${DOCKER_CONFIG_PATH}:/root/.docker/config.json"
    else
        echo "ERROR: File not found [${DOCKER_CONFIG_PATH}], skipping local docker config file"
    fi
fi
IMAGE="${DISTRIBUTION_DOCKER_REPO}/distribution-installer:$DISTRIBUTION_VERSION"

# check if docker images exists locally in order to hide "Unable to find image locally" message
docker inspect ${IMAGE} &>/dev/null || docker pull ${IMAGE}

exec docker run --rm ${DOCKER_RUN_OPTIONS} ${DOCKER_ADDR} ${DISTRIBUTION_VOLUMES} -e DISTRIBUTION_DOCKER_REPO=${DISTRIBUTION_DOCKER_REPO} -e DOCKER_INSTALLER_OFFLINE=${DOCKER_INSTALLER_OFFLINE} -e DISTRIBUTIONDB_DOCKER_REPO=${DISTRIBUTIONDB_DOCKER_REPO} -e DISTRIBUTION_VERSION=${DISTRIBUTION_VERSION} -e SERVER_FLAG=${SERVER_FLAG} -e DISTRIBUTION_MOUNT_ROOT=${DISTRIBUTION_MOUNT_ROOT} -e DOCKER_SERVER_CURRENT_VER=${DOCKER_SERVER_CURRENT_VER} ${IMAGE} "$CMD"

exit $?